@@ -2,7 +2,7 @@
 from copy import deepcopy
 
 from django.contrib import admin
-from django.contrib.admin.options import BaseModelAdmin, InlineModelAdmin
+from django.contrib.admin.options import BaseModelAdmin, flatten_fieldsets, InlineModelAdmin
 from django.contrib.contenttypes import generic
 
 # Ensure that models are registered for translation before TranslationAdmin
@@ -11,7 +11,8 @@ from django.contrib.contenttypes import generic
 import modeltranslation.models  # NOQA
 from modeltranslation.settings import DEFAULT_LANGUAGE
 from modeltranslation.translator import translator
-from modeltranslation.utils import get_translation_fields, build_css_class
+from modeltranslation.utils import (
+    get_translation_fields, build_css_class, build_localized_fieldname, get_language)
 
 
 class TranslationBaseModelAdmin(BaseModelAdmin):
@@ -47,10 +48,12 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
 
         # For every localized field copy the widget from the original field
         # and add a css class to identify a modeltranslation widget.
-        if db_field.name in self.trans_opts.localized_fieldnames_rev:
-            orig_fieldname = self.trans_opts.localized_fieldnames_rev[db_field.name]
-            orig_formfield = self.formfield_for_dbfield(
-                self.model._meta.get_field(orig_fieldname), **kwargs)
+        try:
+            orig_field = db_field.translated_field
+        except AttributeError:
+            pass
+        else:
+            orig_formfield = self.formfield_for_dbfield(orig_field, **kwargs)
             field.widget = deepcopy(orig_formfield.widget)
             css_classes = field.widget.attrs.get('class', '').split(' ')
             css_classes.append('mt')
@@ -62,7 +65,7 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
                 # widget.
                 css_classes.append('mt-default')
                 if (orig_formfield.required or self._orig_was_required.get(
-                        '%s.%s' % (db_field.model._meta, orig_fieldname))):
+                        '%s.%s' % (orig_field.model._meta, orig_field.name))):
                     # In case the original form field was required, make the
                     # default translation field required instead.
                     orig_formfield.required = False
@@ -76,8 +79,8 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
             exclude = tuple()
         if exclude:
             exclude_new = tuple(exclude)
-            return exclude_new + tuple(self.trans_opts.fields)
-        return tuple(self.trans_opts.fields)
+            return exclude_new + tuple(self.trans_opts.fields.keys())
+        return tuple(self.trans_opts.fields.keys())
 
     def replace_orig_field(self, option):
         """
@@ -87,9 +90,9 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
         Returns a new list with replaced fields. If `option` contains no
         registered fields, it is returned unmodified.
 
-        >>> print self.trans_opts.fields
-        ('title',)
-        >>> get_translation_fields(self.trans_opts.fields[0])
+        >>> print self.trans_opts.fields.keys()
+        ['title',]
+        >>> get_translation_fields(self.trans_opts.fields.keys()[0])
         ['title_de', 'title_en']
         >>> self.replace_orig_field(['title', 'url'])
         ['title_de', 'title_en', 'url']
@@ -100,10 +103,6 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
             2. They don't scale well with more than a few languages
             3. It's better than not handling them at all (okay that's weak)
 
-        >>> print self.trans_opts.fields
-        (('title', 'url'), 'email')
-        >>> get_translation_fields(self.trans_opts.fields[0])
-        ['title_de', 'title_en', 'url_de', 'url_en', 'email_de', 'email_en']
         >>> self.replace_orig_field((('title', 'url'), 'email', 'text'))
         ['title_de', 'title_en', 'url_de', 'url_en', 'email_de', 'email_en', 'text']
         """
@@ -131,11 +130,15 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
 
     def _patch_prepopulated_fields(self):
         if self.prepopulated_fields:
+            # TODO: Perhaps allow to configure which language the slug should be based on?
+            lang = get_language()
             prepopulated_fields_new = dict(self.prepopulated_fields)
-            for (k, v) in self.prepopulated_fields.items():
-                if v[0] in self.trans_opts.fields:
-                    translation_fields = get_translation_fields(v[0])
-                    prepopulated_fields_new[k] = tuple([translation_fields[0]])
+            translation_fields = []
+            for k, v in self.prepopulated_fields.items():
+                for i in v:
+                    if i in self.trans_opts.fields.keys():
+                        translation_fields.append(build_localized_fieldname(i, lang))
+                prepopulated_fields_new[k] = tuple(translation_fields)
             self.prepopulated_fields = prepopulated_fields_new
 
     def _do_get_form_or_formset(self, request, obj, **kwargs):
@@ -147,8 +150,7 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
         else:
             exclude = list(self.exclude)
         exclude.extend(self.get_readonly_fields(request, obj))
-        if not self.exclude and hasattr(
-                self.form, '_meta') and self.form._meta.exclude:
+        if not self.exclude and hasattr(self.form, '_meta') and self.form._meta.exclude:
             # Take the custom ModelForm's Meta.exclude into account only if the
             # ModelAdmin doesn't define its own.
             exclude.extend(self.form._meta.exclude)
@@ -187,15 +189,24 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
         if exclude_languages:
             excl_languages = exclude_languages
         exclude = []
-        for orig_fieldname, translation_fields in self.trans_opts.localized_fieldnames.iteritems():
+        for orig_fieldname, translation_fields in self.trans_opts.fields.iteritems():
             for tfield in translation_fields:
-                language = tfield.split('_')[-1]
+                language = tfield.name.split('_')[-1]
                 if language in excl_languages and tfield not in exclude:
                     exclude.append(tfield)
         return tuple(exclude)
 
+    def get_readonly_fields(self, request, obj=None):
+        """
+        Hook for specifying custom readonly fields.
+        """
+        return self.replace_orig_field(self.readonly_fields)
+
 
 class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
+    # TODO: Consider addition of a setting which allows to override the fallback to True
+    group_fieldsets = False
+
     def __init__(self, *args, **kwargs):
         super(TranslationAdmin, self).__init__(*args, **kwargs)
         self._patch_list_editable()
@@ -214,6 +225,46 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
             self.list_editable = editable_new
             self.list_display = display_new
 
+    def _group_fieldsets(self, fieldsets):
+        # Fieldsets are not grouped by default. The function is activated by
+        # setting TranslationAdmin.group_fieldsets to True. If the admin class
+        # already defines a fieldset, we leave it alone and assume the author
+        # has done whatever grouping for translated fields they desire.
+        if not self.declared_fieldsets and self.group_fieldsets is True:
+            flattened_fieldsets = flatten_fieldsets(fieldsets)
+
+            # Create a fieldset to group each translated field's localized fields
+            untranslated_fields = [
+                f.name for f in self.opts.fields if (
+                    # Exclude the primary key field
+                    f is not self.opts.auto_field
+                    # Exclude non-editable fields
+                    and f.editable
+                    # Exclude the translation fields
+                    and not hasattr(f, 'translated_field')
+                    # Honour field arguments. We rely on the fact that the
+                    # passed fieldsets argument is already fully filtered
+                    # and takes options like exclude into account.
+                    and f.name in flattened_fieldsets
+                )
+            ]
+            # TODO: Allow setting a label
+            fieldsets = [('', {'fields': untranslated_fields},)] if untranslated_fields else []
+
+            for orig_field, trans_fields in self.trans_opts.fields.items():
+                trans_fieldnames = [f.name for f in sorted(trans_fields, key=lambda x: x.name)]
+                if any(f in trans_fieldnames for f in flattened_fieldsets):
+                    # Extract the original field's verbose_name for use as this
+                    # fieldset's label - using ugettext_lazy in your model
+                    # declaration can make that translatable.
+                    label = self.model._meta.get_field(orig_field).verbose_name
+                    fieldsets.append((label, {
+                        'fields': trans_fieldnames,
+                        'classes': ('mt-fieldset',)
+                    }))
+
+        return fieldsets
+
     def get_form(self, request, obj=None, **kwargs):
         kwargs = self._do_get_form_or_formset(request, obj, **kwargs)
         return super(TranslationAdmin, self).get_form(request, obj, **kwargs)
@@ -221,9 +272,8 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
     def get_fieldsets(self, request, obj=None):
         if self.declared_fieldsets:
             return self._do_get_fieldsets_pre_form_or_formset()
-        form = self.get_form(request, obj)
-        return self._do_get_fieldsets_post_form_or_formset(
-            request, form, obj)
+        return self._group_fieldsets(
+            self._do_get_fieldsets_post_form_or_formset(request, self.get_form(request, obj), obj))
 
 
 class TranslationInlineModelAdmin(TranslationBaseModelAdmin, InlineModelAdmin):
