@@ -108,6 +108,23 @@ class TranslationField(object):
         # (will show up e.g. in the admin).
         self.verbose_name = build_localized_verbose_name(translated_field.verbose_name, language)
 
+    # Django 1.5 changed definition of __hash__ for fields to be fine with hash requirements.
+    # It spoiled our machinery, since TranslationField has the same creation_counter as its
+    # original field and fields didn't get added to sets.
+    # So here we override __eq__ and __hash__ to fix the issue while retaining fine with
+    # http://docs.python.org/2.7/reference/datamodel.html#object.__hash__
+    def __eq__(self, other):
+        if isinstance(other, fields.Field):
+            return (self.creation_counter == other.creation_counter and
+                    self.language == getattr(other, 'language', None))
+        return super(TranslationField, self).__eq__(other)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __hash__(self):
+        return hash((self.creation_counter, self.language))
+
     def get_attname_column(self):
         attname = self.get_attname()
         column = build_localized_fieldname(
@@ -156,7 +173,7 @@ class TranslationFieldDescriptor(object):
             # Here we check only for None and '', because e.g. 0 should not fall back.
             if val is not None and val != '':
                 return val
-        if self.fallback_value is None:
+        if self.fallback_value is None or not mt_settings.ENABLE_FALLBACKS:
             return self.field.get_default()
         else:
             return self.fallback_value
