@@ -1,16 +1,9 @@
 # -*- coding: utf-8 -*-
-from warnings import warn
-
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 
 
 TRANSLATION_FILES = tuple(getattr(settings, 'MODELTRANSLATION_TRANSLATION_FILES', ()))
-TRANSLATION_REGISTRY = getattr(settings, 'MODELTRANSLATION_TRANSLATION_REGISTRY', None)
-if TRANSLATION_REGISTRY:
-    TRANSLATION_FILES += (TRANSLATION_REGISTRY,)
-    warn('The setting MODELTRANSLATION_TRANSLATION_REGISTRY is deprecated, '
-         'use MODELTRANSLATION_TRANSLATION_FILES instead.', DeprecationWarning)
 
 AVAILABLE_LANGUAGES = [l[0] for l in settings.LANGUAGES]
 DEFAULT_LANGUAGE = getattr(settings, 'MODELTRANSLATION_DEFAULT_LANGUAGE', None)
@@ -51,3 +44,4 @@ for key, value in FALLBACK_LANGUAGES.iteritems():
         if lang not in AVAILABLE_LANGUAGES:
             raise ImproperlyConfigured(
                 'MODELTRANSLATION_FALLBACK_LANGUAGES: "%s" not in LANGUAGES setting.' % lang)
+ENABLE_FALLBACKS = getattr(settings, 'MODELTRANSLATION_ENABLE_FALLBACKS', True)
